#! /bin/bash
#
# Edouard.Thiel@univ-amu.fr - 29/12/2013
#
# This script allows to control the brightness of a monitor HP Dreamcolor2.
# It needs packages ddccontrol and ddccontrol-db
# The monitor is described in /usr/share/ddccontrol-db/monitor/LGD02FC.xml
#
# This program is free software under the terms of the
# GNU Lesser General Public License (LGPL) version 2.1.

# Monitor-dependent parameters
#
# To get the i2c device number, try: 
#   $ sudo LANG=C ddccontrol -p | grep Device:
# If your monitor is not in the database, create a new entry in
# /usr/share/ddccontrol-db/monitor/ using the ouput of (replace x)
#   $ ddccontrol -c dev:/dev/i2c-x

DEV_I2C="/dev/i2c-9"
DDC_QUERY="0x10"
VALUE_FILE="/usr/local/dreamcolor2/current_value"
BRIGHT_MIN=40
BRIGHT_MAX=300
BRIGHT_DFT=80


check_range ()
{
    local val="$1"
    if ((val < BRIGHT_MIN || val > BRIGHT_MAX))
    then
        echo "Value out of range" >&2
        return 1
    fi
    return 0
}

set_bright ()
{
    local val="$1"
    check_range "$val" || return
    ddccontrol -r "$DDC_QUERY" -w "$val" "dev:$DEV_I2C" > /dev/null
}

get_current_value ()
{
    if [ -r "$VALUE_FILE" ]
    then echo "$(< $VALUE_FILE)"
    else echo "$BRIGHT_DFT"
    fi
}

set_current_value ()
{
    local val="$1"
    check_range "$val" || return
    echo "$val" >| "$VALUE_FILE"
}

percent_to_value ()
{
    local per="$1"
    if ((per < 0||per>100)); then
        echo "Percentage out of range" >&2
        return 1
    fi
    echo $((BRIGHT_MIN+(BRIGHT_MAX-BRIGHT_MIN)*per/100))
}

value_to_percent ()
{
    local val="$1"
    if ! check_range "$val" ; then return 1 ; fi
    echo $(((val-BRIGHT_MIN)*100/(BRIGHT_MAX-BRIGHT_MIN)))
    return 0
}

notify_brightness ()
{
    local val="$1"
    local per=$(value_to_percent "$val")
    zenity --progress --timeout 2 --no-cancel --title="Brightness" \
        --text "$val cd/m2            $per%" \
        --percentage="$per" --window-icon=info &
}

dbus_send_brightness ()
{
    local val="$1"
    local per=$(value_to_percent "$val")

    dbus-send --session --print-reply --dest="org.gnome.SettingsDaemon" \
        /org/gnome/SettingsDaemon/Power \
        org.gnome.SettingsDaemon.Power.Screen.SetPercentage "uint32:$per" > /dev/null
}


# Main

if (($# < 1)); then
    cat << EOF
USAGE: $0 actions

  Actions:
    --get-cur   : get current brightness
    --res-cur   : restore current brightness
    --set-val v : set brightness v
    --set-per p : set percentage p
    --inc-val s : increase value by s
    --dec-val s : decrease value by s
    --notify    : notify new value and percentage in a popup
    --send-dbus : send new percentage to dbus
EOF
exit 1
fi

while (($# >= 1)) ; do
    case "$1" in
        --get-cur)  get_current_value 
                    shift 1 ;;
        --res-cur)  v=$(get_current_value)
                    set_bright "$v" 
                    shift 1 ;;
        --set-val)  v="${2-}"
                    set_bright "$v" && set_current_value "$v" 
                    shift 2 ;;
        --set-per)  v=$(percent_to_value "${2-}")
                    set_bright "$v" && set_current_value "$v"
                    shift 2 ;;
        --inc-val)  s="${2-}" ; v=$(get_current_value) ; v=$((v+s))
                    set_bright "$v" && set_current_value "$v" 
                    shift 2 ;;
        --dec-val)  s="${2-}" ; v=$(get_current_value) ; v=$((v-s))
                    set_bright "$v" && set_current_value "$v" 
                    shift 2 ;;
        --notify)   notify_brightness $(get_current_value)
                    shift 1 ;;
        --send-dbus) dbus_send_brightness $(get_current_value)
                    shift 1 ;;
        *) echo "$0: syntax error" >&2 ; exit 1 ;;
    esac
done

exit 0


